@@ -35,14 +35,6 @@
  * Silicon Graphics, Inc.
  */
 
-typedef struct __GLXtextureFromPixmap __GLXtextureFromPixmap;
-struct __GLXtextureFromPixmap {
-    int (*bindTexImage) (__GLXcontext * baseContext,
-                         int buffer, __GLXdrawable * pixmap);
-    int (*releaseTexImage) (__GLXcontext * baseContext,
-                            int buffer, __GLXdrawable * pixmap);
-};
-
 struct __GLXcontext {
     void (*destroy) (__GLXcontext * context);
     int (*makeCurrent) (__GLXcontext * context);
@@ -50,7 +42,11 @@ struct __GLXcontext {
     int (*copy) (__GLXcontext * dst, __GLXcontext * src, unsigned long mask);
     Bool (*wait) (__GLXcontext * context, __GLXclientState * cl, int *error);
 
-    __GLXtextureFromPixmap *textureFromPixmap;
+    /* EXT_texture_from_pixmap */
+    int (*bindTexImage) (__GLXcontext * baseContext,
+                         int buffer, __GLXdrawable * pixmap);
+    int (*releaseTexImage) (__GLXcontext * baseContext,
+                            int buffer, __GLXdrawable * pixmap);
 
     /*
      ** list of context structs
@@ -68,6 +64,11 @@ struct __GLXcontext {
      */
     __GLXscreen *pGlxScreen;
 
+    /*
+     ** If this context is current for a client, this will be that client
+     */
+    ClientPtr currentClient;
+
     /*
      ** The XID of this context.
      */
@@ -83,21 +84,11 @@ struct __GLXcontext {
      */
     GLboolean idExists;
 
-    /*
-     ** Whether this context is current for some client.
-     */
-    GLboolean isCurrent;
-
     /*
      ** Whether this context is a direct rendering context.
      */
     GLboolean isDirect;
 
-    /*
-     ** This flag keeps track of whether there are unflushed GL commands.
-     */
-    GLboolean hasUnflushedCommands;
-
     /*
      ** Current rendering mode for this context.
      */
@@ -108,6 +99,11 @@ struct __GLXcontext {
      */
     GLenum resetNotificationStrategy;
 
+    /**
+     * Context release behavior
+     */
+    GLenum releaseBehavior;
+
     /*
      ** Buffers for feedback and selection.
      */
@@ -116,6 +112,16 @@ struct __GLXcontext {
     GLuint *selectBuf;
     GLint selectBufSize;        /* number of elements allocated */
 
+    /*
+     ** Keep track of large rendering commands, which span multiple requests.
+     */
+    GLint largeCmdBytesSoFar;   /* bytes received so far        */
+    GLint largeCmdBytesTotal;   /* total bytes expected         */
+    GLint largeCmdRequestsSoFar;        /* requests received so far     */
+    GLint largeCmdRequestsTotal;        /* total requests expected      */
+    GLbyte *largeCmdBuf;
+    GLint largeCmdBufSize;
+
     /*
      ** The drawable private this context is bound to
      */
