@@ -57,6 +57,60 @@ struct xwl_format {
 
 struct xwl_pixmap;
 struct xwl_window;
+struct xwl_screen;
+
+struct xwl_egl_backend {
+    /* Set by the backend if available */
+    Bool is_available;
+
+    /* Called once for each interface in the global registry. Backends
+     * should use this to bind to any wayland interfaces they need.
+     */
+    Bool (*init_wl_registry)(struct xwl_screen *xwl_screen,
+                             struct wl_registry *wl_registry,
+                             uint32_t id, const char *name,
+                             uint32_t version);
+
+    /* Check that the required Wayland interfaces are available.
+     */
+    Bool (*has_wl_interfaces)(struct xwl_screen *xwl_screen);
+
+    /* Called before glamor has been initialized. Backends should setup a
+     * valid, glamor compatible EGL context in this hook.
+     */
+    Bool (*init_egl)(struct xwl_screen *xwl_screen);
+
+    /* Called after glamor has been initialized, and after all of the
+     * common Xwayland DDX hooks have been connected. Backends should use
+     * this to setup any required wraps around X server callbacks like
+     * CreatePixmap.
+     */
+     Bool (*init_screen)(struct xwl_screen *xwl_screen);
+
+     /* Called by Xwayland to retrieve a pointer to a valid wl_buffer for
+      * the given window/pixmap combo so that damage to the pixmap may be
+      * displayed on-screen. Backends should use this to create a new
+      * wl_buffer for a currently buffer-less pixmap, or simply return the
+      * pixmap they've prepared beforehand.
+      */
+     struct wl_buffer *(*get_wl_buffer_for_pixmap)(PixmapPtr pixmap,
+                                                   Bool *created);
+
+     /* Called by Xwayland to perform any pre-wl_surface damage routines
+      * that are required by the backend. If your backend is poorly
+      * designed and lacks the ability to render directly to a surface,
+      * you should implement blitting from the glamor pixmap to the wayland
+      * pixmap here. Otherwise, this callback is optional.
+      */
+     void (*post_damage)(struct xwl_window *xwl_window,
+                         PixmapPtr pixmap, RegionPtr region);
+
+     /* Called by Xwayland to confirm with the egl backend that the given
+      * pixmap is completely setup and ready for display on-screen. This
+      * callback is optional.
+      */
+     Bool (*allow_commits)(struct xwl_window *xwl_window);
+};
 
 struct xwl_screen {
     int width;
@@ -77,7 +131,9 @@ struct xwl_screen {
     CloseScreenProcPtr CloseScreen;
     RealizeWindowProcPtr RealizeWindow;
     UnrealizeWindowProcPtr UnrealizeWindow;
+    DestroyWindowProcPtr DestroyWindow;
     XYToWindowProcPtr XYToWindow;
+    SetWindowPixmapProcPtr SetWindowPixmap;
 
     struct xorg_list output_list;
     struct xorg_list seat_list;
@@ -108,55 +164,10 @@ struct xwl_screen {
     struct xwl_format *formats;
     void *egl_display, *egl_context;
 
-    /* the current backend for creating pixmaps on wayland */
-    struct {
-        /* Called once for each interface in the global registry. Backends
-         * should use this to bind to any wayland interfaces they need. This
-         * callback is optional.
-         */
-        void (*init_wl_registry)(struct xwl_screen *xwl_screen,
-                                 struct wl_registry *wl_registry,
-                                 const char *name, uint32_t id,
-                                 uint32_t version);
-
-        /* Called before glamor has been initialized. Backends should setup a
-         * valid, glamor compatible EGL context in this hook.
-         */
-        Bool (*init_egl)(struct xwl_screen *xwl_screen);
-
-        /* Called after glamor has been initialized, and after all of the
-         * common Xwayland DDX hooks have been connected. Backends should use
-         * this to setup any required wraps around X server callbacks like
-         * CreatePixmap.
-         */
-        Bool (*init_screen)(struct xwl_screen *xwl_screen);
-
-        /* Called by Xwayland to retrieve a pointer to a valid wl_buffer for
-         * the given window/pixmap combo so that damage to the pixmap may be
-         * displayed on-screen. Backends should use this to create a new
-         * wl_buffer for a currently buffer-less pixmap, or simply return the
-         * pixmap they've prepared beforehand.
-         */
-        struct wl_buffer *(*get_wl_buffer_for_pixmap)(PixmapPtr pixmap,
-                                                      unsigned short width,
-                                                      unsigned short height,
-                                                      Bool *created);
-
-        /* Called by Xwayland to perform any pre-wl_surface damage routines
-         * that are required by the backend. If your backend is poorly
-         * designed and lacks the ability to render directly to a surface,
-         * you should implement blitting from the glamor pixmap to the wayland
-         * pixmap here. Otherwise, this callback is optional.
-         */
-        void (*post_damage)(struct xwl_window *xwl_window,
-                            PixmapPtr pixmap, RegionPtr region);
-
-        /* Called by Xwayland to confirm with the egl backend that the given
-         * pixmap is completely setup and ready for display on-screen. This
-         * callback is optional.
-         */
-        Bool (*allow_commits)(struct xwl_window *xwl_window);
-    } egl_backend;
+    struct xwl_egl_backend gbm_backend;
+    struct xwl_egl_backend eglstream_backend;
+    /* pointer to the current backend for creating pixmaps on wayland */
+    struct xwl_egl_backend *egl_backend;
 
     struct glamor_context *glamor_ctx;
 
@@ -168,30 +179,33 @@ struct xwl_window {
     struct wl_surface *surface;
     struct wl_shell_surface *shell_surface;
     WindowPtr window;
-    DamagePtr damage;
     struct xorg_list link_damage;
     struct wl_callback *frame_callback;
     Bool allow_commits;
 #ifdef GLAMOR_HAS_GBM
-    /* present */
-    RRCrtcPtr present_crtc_fake;
-    struct xorg_list present_link;
-    WindowPtr present_window;
-    uint64_t present_msc;
-    uint64_t present_ust;
-
-    OsTimerPtr present_timer;
-    Bool present_timer_firing;
-
-    struct wl_callback *present_frame_callback;
-    struct wl_callback *present_sync_callback;
-
-    struct xorg_list present_event_list;
-    struct xorg_list present_release_queue;
+    struct xorg_list frame_callback_list;
+    Bool present_flipped;
 #endif
 };
 
 #ifdef GLAMOR_HAS_GBM
+struct xwl_present_window {
+    struct xwl_screen *xwl_screen;
+    struct xwl_present_event *sync_flip;
+    WindowPtr window;
+    struct xorg_list frame_callback_list;
+
+    uint64_t msc;
+    uint64_t ust;
+
+    OsTimerPtr frame_timer;
+
+    struct wl_callback *sync_callback;
+
+    struct xorg_list event_list;
+    struct xorg_list release_queue;
+};
+
 struct xwl_present_event {
     uint64_t event_id;
     uint64_t target_msc;
@@ -200,8 +214,7 @@ struct xwl_present_event {
     Bool pending;
     Bool buffer_released;
 
-    WindowPtr present_window;
-    struct xwl_window *xwl_window;
+    struct xwl_present_window *xwl_present_window;
     struct wl_buffer *buffer;
 
     struct xorg_list list;
@@ -261,6 +274,7 @@ struct xwl_seat {
     char *keymap;
     struct wl_surface *keyboard_focus;
 
+    struct xorg_list axis_discrete_pending;
     struct xorg_list sync_pending;
 
     struct xwl_pointer_warp_emulator *pointer_warp_emulator;
@@ -299,13 +313,13 @@ struct xwl_tablet_tool {
 
     DeviceIntPtr xdevice;
     uint32_t proximity_in_serial;
-    uint32_t x;
-    uint32_t y;
+    double x;
+    double y;
     uint32_t pressure;
-    float tilt_x;
-    float tilt_y;
-    float rotation;
-    float slider;
+    double tilt_x;
+    double tilt_y;
+    double rotation;
+    double slider;
 
     uint32_t buttons_now,
              buttons_prev;
@@ -363,6 +377,8 @@ struct xwl_output {
     Bool xdg_output_done;
 };
 
+void xwl_window_create_frame_callback(struct xwl_window *xwl_window);
+
 void xwl_sync_events (struct xwl_screen *xwl_screen);
 
 Bool xwl_screen_init_cursor(struct xwl_screen *xwl_screen);
@@ -412,7 +428,11 @@ PixmapPtr xwl_shm_create_pixmap(ScreenPtr screen, int width, int height,
 Bool xwl_shm_destroy_pixmap(PixmapPtr pixmap);
 struct wl_buffer *xwl_shm_pixmap_get_wl_buffer(PixmapPtr pixmap);
 
-
+#ifdef XWL_HAS_GLAMOR
+void xwl_glamor_init_backends(struct xwl_screen *xwl_screen,
+                              Bool use_eglstream);
+void xwl_glamor_select_backend(struct xwl_screen *xwl_screen,
+                               Bool use_eglstream);
 Bool xwl_glamor_init(struct xwl_screen *xwl_screen);
 
 Bool xwl_screen_set_drm_interface(struct xwl_screen *xwl_screen,
@@ -420,58 +440,53 @@ Bool xwl_screen_set_drm_interface(struct xwl_screen *xwl_screen,
 Bool xwl_screen_set_dmabuf_interface(struct xwl_screen *xwl_screen,
                                      uint32_t id, uint32_t version);
 struct wl_buffer *xwl_glamor_pixmap_get_wl_buffer(PixmapPtr pixmap,
-                                                  unsigned short width,
-                                                  unsigned short height,
                                                   Bool *created);
 void xwl_glamor_init_wl_registry(struct xwl_screen *xwl_screen,
                                  struct wl_registry *registry,
                                  uint32_t id, const char *interface,
                                  uint32_t version);
+Bool xwl_glamor_has_wl_interfaces(struct xwl_screen *xwl_screen,
+                                 struct xwl_egl_backend *xwl_egl_backend);
 void xwl_glamor_post_damage(struct xwl_window *xwl_window,
                             PixmapPtr pixmap, RegionPtr region);
 Bool xwl_glamor_allow_commits(struct xwl_window *xwl_window);
+void xwl_glamor_egl_make_current(struct xwl_screen *xwl_screen);
 
 #ifdef GLAMOR_HAS_GBM
+void xwl_present_frame_callback(struct xwl_present_window *xwl_present_window);
 Bool xwl_present_init(ScreenPtr screen);
-void xwl_present_cleanup(struct xwl_window *xwl_window, WindowPtr window);
-#endif
-
-void xwl_screen_release_tablet_manager(struct xwl_screen *xwl_screen);
-
-void xwl_output_get_xdg_output(struct xwl_output *xwl_output);
-void xwl_screen_init_xdg_output(struct xwl_screen *xwl_screen);
-
-void xwl_glamor_egl_make_current(struct xwl_screen *xwl_screen);
-Bool xwl_glamor_egl_supports_device_probing(void);
-void **xwl_glamor_egl_get_devices(int *num_devices);
-Bool xwl_glamor_egl_device_has_egl_extensions(void *device,
-                                              const char **ext_list,
-                                              size_t size);
+void xwl_present_cleanup(WindowPtr window);
+void xwl_present_unrealize_window(struct xwl_present_window *xwl_present_window);
+#endif /* GLAMOR_HAS_GBM */
 
 #ifdef XV
 /* glamor Xv Adaptor */
 Bool xwl_glamor_xv_init(ScreenPtr pScreen);
-#endif
+#endif /* XV */
+
+#endif /* XWL_HAS_GLAMOR */
+
+void xwl_screen_release_tablet_manager(struct xwl_screen *xwl_screen);
+
+void xwl_screen_init_xdg_output(struct xwl_screen *xwl_screen);
 
 #ifdef XF86VIDMODE
 void xwlVidModeExtensionInit(void);
 #endif
 
 #ifdef GLAMOR_HAS_GBM
-Bool xwl_glamor_init_gbm(struct xwl_screen *xwl_screen);
+void xwl_glamor_init_gbm(struct xwl_screen *xwl_screen);
 #else
-static inline Bool xwl_glamor_init_gbm(struct xwl_screen *xwl_screen)
+static inline void xwl_glamor_init_gbm(struct xwl_screen *xwl_screen)
 {
-    return FALSE;
 }
 #endif
 
 #ifdef XWL_HAS_EGLSTREAM
-Bool xwl_glamor_init_eglstream(struct xwl_screen *xwl_screen);
+void xwl_glamor_init_eglstream(struct xwl_screen *xwl_screen);
 #else
-static inline Bool xwl_glamor_init_eglstream(struct xwl_screen *xwl_screen)
+static inline void xwl_glamor_init_eglstream(struct xwl_screen *xwl_screen)
 {
-    return FALSE;
 }
 #endif
 
