@@ -40,6 +40,7 @@
 #define _WINDOWSWM_SERVER_
 #include <X11/extensions/windowswmstr.h>
 #include "winmultiwindowclass.h"
+#include "winmultiwindowicons.h"
 #include <X11/Xatom.h>
 
 /*
@@ -516,12 +517,6 @@ winMWExtWMRestackFrame(RootlessFrameID wid, RootlessFrameID nextWid)
     win32RootlessWindowPtr pRLNextWinPriv = (win32RootlessWindowPtr) nextWid;
 
     winScreenPriv(pRLWinPriv->pFrame->win->drawable.pScreen);
-    winScreenInfo *pScreenInfo = NULL;
-    DWORD dwCurrentProcessID = GetCurrentProcessId();
-    DWORD dwWindowProcessID = 0;
-    HWND hWnd;
-    Bool fFirst = TRUE;
-    Bool fNeedRestack = TRUE;
 
 #if CYGMULTIWINDOW_DEBUG
     winDebug("winMWExtWMRestackFrame (%p)\n", pRLWinPriv);
@@ -530,9 +525,6 @@ winMWExtWMRestackFrame(RootlessFrameID wid, RootlessFrameID nextWid)
     if (pScreenPriv && pScreenPriv->fRestacking)
         return;
 
-    if (pScreenPriv)
-        pScreenInfo = pScreenPriv->pScreenInfo;
-
     pRLWinPriv->fRestackingNow = TRUE;
 
     /* Show window */
@@ -541,68 +533,12 @@ winMWExtWMRestackFrame(RootlessFrameID wid, RootlessFrameID nextWid)
 
     if (pRLNextWinPriv == NULL) {
 #if CYGMULTIWINDOW_DEBUG
-        winDebug("Win %08x is top\n", pRLWinPriv);
+        winDebug("Win %p is top\n", pRLWinPriv);
 #endif
         pScreenPriv->widTop = wid;
         SetWindowPos(pRLWinPriv->hWnd, HWND_TOP,
                      0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE | SWP_NOACTIVATE);
     }
-    else if (winIsInternalWMRunning(pScreenInfo)) {
-        /* using mulwinidow wm */
-#if CYGMULTIWINDOW_DEBUG
-        winDebug("Win %08x is not top\n", pRLWinPriv);
-#endif
-        for (hWnd = GetNextWindow(pRLWinPriv->hWnd, GW_HWNDPREV);
-             fNeedRestack && hWnd != NULL;
-             hWnd = GetNextWindow(hWnd, GW_HWNDPREV)) {
-            GetWindowThreadProcessId(hWnd, &dwWindowProcessID);
-
-            if ((dwWindowProcessID == dwCurrentProcessID)
-                && GetProp(hWnd, WIN_WINDOW_PROP)) {
-                if (hWnd == pRLNextWinPriv->hWnd) {
-                    /* Enable interleave X window and Windows window */
-                    if (!fFirst) {
-#if CYGMULTIWINDOW_DEBUG
-                        winDebug("raise: Insert after Win %08x\n",
-                                 pRLNextWinPriv);
-#endif
-                        SetWindowPos(pRLWinPriv->hWnd, pRLNextWinPriv->hWnd,
-                                     0, 0, 0, 0,
-                                     SWP_NOMOVE | SWP_NOSIZE | SWP_NOACTIVATE);
-                    }
-                    else {
-#if CYGMULTIWINDOW_DEBUG
-                        winDebug("No change\n");
-#endif
-                    }
-                    fNeedRestack = FALSE;
-                    break;
-                }
-                if (fFirst)
-                    fFirst = FALSE;
-            }
-        }
-
-        for (hWnd = GetNextWindow(pRLWinPriv->hWnd, GW_HWNDNEXT);
-             fNeedRestack && hWnd != NULL;
-             hWnd = GetNextWindow(hWnd, GW_HWNDNEXT)) {
-            GetWindowThreadProcessId(hWnd, &dwWindowProcessID);
-
-            if ((dwWindowProcessID == dwCurrentProcessID)
-                && GetProp(hWnd, WIN_WINDOW_PROP)) {
-                if (hWnd == pRLNextWinPriv->hWnd) {
-#if CYGMULTIWINDOW_DEBUG
-                    winDebug("lower: Insert after Win %08x\n", pRLNextWinPriv);
-#endif
-                    SetWindowPos(pRLWinPriv->hWnd, pRLNextWinPriv->hWnd,
-                                 0, 0, 0, 0,
-                                 SWP_NOMOVE | SWP_NOSIZE | SWP_NOACTIVATE);
-                    fNeedRestack = FALSE;
-                    break;
-                }
-            }
-        }
-    }
     else {
         /* using general wm like twm, wmaker etc.
            Interleave X window and Windows window will cause problem. */
